"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.VanillaEventBus = void 0;
const tslib_1 = require("tslib");
const EventBusValidator_1 = require("./EventBusValidator");
const eventbus_utils_1 = require("./eventbus-utils");
class VanillaEventBus {
    constructor(config) {
        this.config = config;
        this.init = () => (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            if (this.config.publisher && this.config.publisher.publishInit) {
                const topicnames = (0, eventbus_utils_1.getRootQueryNames)(this.config.publisher.schema);
                yield this.config.publisher.publishInit(topicnames);
            }
            if (this.config.subscriber) {
                yield this.consume(this.config.subscriber);
            }
        });
        this.consume = (args) => (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            if (!this.consumeValidator) {
                throw new Error("Subscriber config not added");
            }
            yield this.consumeValidator
                .validateConsumerQueries(args.queries)
                .then((topics) => {
                topics.forEach((topicName) => (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
                    yield args.getSubscription(topicName, (data, ...rest) => (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
                        var _a, _b, _c;
                        let consumeEnd = [];
                        (_a = this.config.plugins) === null || _a === void 0 ? void 0 : _a.forEach((plugin) => {
                            if (plugin.consumeStart) {
                                const foo = plugin.consumeStart({ topic: topicName, data }, ...rest);
                                if (foo) {
                                    consumeEnd.push(foo);
                                }
                            }
                        });
                        try {
                            const eventData = yield ((_b = this.consumeValidator) === null || _b === void 0 ? void 0 : _b.extractData(topicName, data));
                            yield args.cb({ key: topicName, data: eventData }, ...rest);
                        }
                        catch (e) {
                            (_c = this.config.plugins) === null || _c === void 0 ? void 0 : _c.forEach((plugin) => {
                                plugin.consumeError &&
                                    plugin.consumeError({ topic: topicName, data }, e);
                            });
                        }
                        consumeEnd === null || consumeEnd === void 0 ? void 0 : consumeEnd.forEach((f) => {
                            f({ topic: topicName, data }, ...rest);
                        });
                    }));
                }));
            });
        });
        this.publish = (props, ...rest) => (0, tslib_1.__awaiter)(this, void 0, void 0, function* () {
            var _a, _b, _c;
            if (!this.publishValidator) {
                throw new Error("Publish config not added!");
            }
            let publishEnd = [];
            (_a = this.config.plugins) === null || _a === void 0 ? void 0 : _a.forEach((plugin) => {
                if (plugin.publishStart) {
                    const foo = plugin.publishStart({ topic: props.topicName, data: props.data }, ...rest);
                    if (foo) {
                        publishEnd.push(foo);
                    }
                }
            });
            try {
                yield this.publishValidator.publishValidate(props);
                yield ((_b = this.config.publisher) === null || _b === void 0 ? void 0 : _b.publish({
                    topicName: props.topicName,
                    data: props.data,
                }, ...rest));
            }
            catch (e) {
                (_c = this.config.plugins) === null || _c === void 0 ? void 0 : _c.forEach((plugin) => {
                    plugin.publishError &&
                        plugin.publishError({
                            data: props.data,
                            topic: props.topicName,
                        }, e);
                });
                publishEnd === null || publishEnd === void 0 ? void 0 : publishEnd.forEach((f) => {
                    f({ topic: props.topicName, data: props.data }, ...rest);
                });
                throw e;
            }
            return;
        });
        if (config.publisher) {
            this.publishValidator = new EventBusValidator_1.EventBusValidator({
                publisherSchema: config.publisher.schema,
            });
        }
        if (config.subscriber) {
            this.consumeValidator = new EventBusValidator_1.EventBusValidator({
                publisherSchema: config.subscriber.schema,
            });
        }
    }
}
exports.VanillaEventBus = VanillaEventBus;
